@@ -1,11 +1,11 @@
 from argparse import ArgumentParser
-from difflib import _mdiff as mdiff
+from difflib import _mdiff as mdiff, SequenceMatcher
 from html import escape
 from re import compile
 from string import whitespace
 from sys import exit, stderr, stdout
 
-VERSION = '0.0.2'
+VERSION = '0.2.0'
 
 SKIPS = [
     compile(r'^.*\[?[Pp]age [0-9ivx]+\]?[ \t\f]*$'),
@@ -50,10 +50,18 @@ HTML = """
       .insert {{ background-color: #8FF; }}
       .change {{ background-color: gray; }}
       .header {{ background-color: orange; }}
+      .w-delete {{
+        color: #F00;
+        text-decoration: line-through;
+      }}
+      .w-insert {{
+        color: #008000;
+        font-weight: bold;
+      }}
     </style>
   </head>
   <body>
-    {table}
+    {output}
   </body>
 </html>"""
 
@@ -130,12 +138,29 @@ WHITESPACES = ''.join([
     '\uFEFF'])   # zero width non-breaking space
 
 
-def cleanup(lines):
+def cleanup(lines, skip_whitespaces):
+    '''Removes skippable content, shrinks multiple empty lines
+    If a link only contains WHITESPACES,
+    that line will get stripped of any WHITESPACES.'''
+
     id_lines = []
-    previous_blank = False
-    for line in lines:
-        if len(line.strip(WHITESPACES)) > 0:
-            previous_blank = False
+    if skip_whitespaces:
+        previous_blank = False
+        for line in lines:
+            if len(line.strip(WHITESPACES)) > 0:
+                previous_blank = False
+                keep = True
+                for skip in SKIPS:
+                    if skip.match(line):
+                        keep = False
+                        break
+                if keep:
+                    id_lines.append(line)
+            elif not previous_blank:
+                id_lines.append(line.strip(WHITESPACES))
+                previous_blank = True
+    else:
+        for line in lines:
             keep = True
             for skip in SKIPS:
                 if skip.match(line):
@@ -143,13 +168,11 @@ def cleanup(lines):
                     break
             if keep:
                 id_lines.append(line)
-        elif not previous_blank:
-            id_lines.append(line.strip(WHITESPACES))
-            previous_blank = True
     return id_lines
 
 
 def add_span(line, css_class):
+    '''Add span tag with the class'''
     stripped_line = line.strip('\0+-^\1').strip(WHITESPACES)
     if len(stripped_line) == 0:
         return ''
@@ -161,38 +184,35 @@ def add_span(line, css_class):
                     replace('\1', '</span>')
 
 
-def main():
-    parser = ArgumentParser(description='Internet-Draft diff tool')
-    parser.add_argument('-t', '--table', action='store_true', default=False,
-                        help='produce a HTML table (default)')
-    parser.add_argument('-c', '--context', action='store_true', default=False,
-                        help='produce a context (default)')
-    parser.add_argument('-l', '--lines', type=int, default=8,
-                        help='set number of context lines (default 8)')
-    parser.add_argument('file1')
-    parser.add_argument('file2')
-    parser.add_argument('--version', action='version',
-                        version='iddiff {}'.format(VERSION))
-    options = parser.parse_args()
-
-    file1 = options.file1
-    file2 = options.file2
-    if options.context:
-        context_lines = options.lines
-    else:
-        context_lines = None
-
-    try:
-        with open(file1, 'r') as file:
-            id_a_lines = cleanup(file.readlines())
-        with open(file2, 'r') as file:
-            id_b_lines = cleanup(file.readlines())
-    except FileNotFoundError as e:
-        stderr.write('iddiff: {}.\n'.format(e))
-        exit(2)
-
+def get_wdiff(first_id_lines, second_id_lines):
+    '''Return wdiff'''
     rows = ''
-    diffs = mdiff(id_a_lines, id_b_lines, context=context_lines)
+    seq = SequenceMatcher(isjunk=None,
+                          a=first_id_lines,
+                          b=second_id_lines)
+    for tag, f1, f2, s1, s2 in seq.get_opcodes():
+        if tag == 'equal':
+            rows += first_id_lines[f1:f2]
+        elif tag == 'delete':
+            rows += '<span class="w-delete">{}</span>'.format(
+                        first_id_lines[f1:f2])
+        elif tag == 'replace':
+            rows += '<span class="w-delete">{}</span>'.format(
+                        first_id_lines[f1:f2])
+            rows += '<span class="w-insert">{}</span>'.format(
+                        second_id_lines[s1:s2])
+        elif tag == 'insert':
+            rows += '<span class="w-insert">{}</span>'.format(
+                        second_id_lines[s1:s2])
+    output = '<pre>{}</pre>'.format(rows)
+
+    return HTML.format(output=output)
+
+
+def get_diff_rows(first_id_lines, second_id_lines, context):
+    '''Retuns diff rows'''
+    rows = ''
+    diffs = mdiff(first_id_lines, second_id_lines, context=context)
     contexts = 0
     for lb, rb, different in diffs:
         if not lb or not rb:
@@ -209,14 +229,85 @@ def main():
                 rows += CHANGED_ROW.format(lline=lline, rline=rline)
     rows.replace('\t', '&nbsp;')
 
-    table = TABLE.format(rows=rows,
-                         filename1=escape(file1),
-                         filename2=escape(file2))
+    return rows
 
-    if options.table:
-        stdout.writelines(table)
+
+def get_html_table(filename1, filename2, rows):
+    '''Return HTML table'''
+    return TABLE.format(filename1=escape(filename1),
+                        filename2=escape(filename2),
+                        rows=rows)
+
+
+def get_iddiff(file1, file2, context_lines=None, table_only=False,
+               wdiff=False, skip_whitespaces=False):
+    '''Return iddiff output'''
+
+    if wdiff:
+        with open(file1, 'r') as file:
+            id_a_lines = ''.join(cleanup(file.readlines(), skip_whitespaces))
+        with open(file2, 'r') as file:
+            id_b_lines = ''.join(cleanup(file.readlines(), skip_whitespaces))
+
+        output = get_wdiff(id_a_lines, id_b_lines)
     else:
-        stdout.writelines(HTML.format(table=table))
+        with open(file1, 'r') as file:
+            id_a_lines = cleanup(file.readlines(), skip_whitespaces)
+        with open(file2, 'r') as file:
+            id_b_lines = cleanup(file.readlines(), skip_whitespaces)
+
+        rows = get_diff_rows(id_a_lines, id_b_lines, context_lines)
+
+        output = get_html_table(file1, file2, rows)
+
+        if not table_only:
+            output = HTML.format(output=output)
+
+    return output
+
+
+def parse_args(args=None):
+    '''Parse command line arguments and return options'''
+    parser = ArgumentParser(description='Internet-Draft diff tool')
+    parser.add_argument('-t', '--table', action='store_true', default=False,
+                        help='produce a HTML table (default)')
+    parser.add_argument('-c', '--context', action='store_true', default=False,
+                        help='produce a context (default)')
+    parser.add_argument('-l', '--lines', type=int, default=8,
+                        help='set number of context lines (default 8)')
+    parser.add_argument('-w', '--wdiff', action='store_true', default=False,
+                        help='produce word difference')
+    parser.add_argument('-s', '--skip-whitespaces', action='store_true',
+                        default=False,
+                        help='skip multilines with only whitespaces')
+    parser.add_argument('file1')
+    parser.add_argument('file2')
+    parser.add_argument('--version', action='version',
+                        version='iddiff {}'.format(VERSION))
+    return parser.parse_args(args)
+
+
+def main():
+    options = parse_args()
+
+    file1 = options.file1
+    file2 = options.file2
+    if options.context:
+        context_lines = options.lines
+    else:
+        context_lines = None
+
+    try:
+        iddiff = get_iddiff(file1=file1,
+                            file2=file2,
+                            context_lines=context_lines,
+                            table_only=options.table,
+                            wdiff=options.wdiff,
+                            skip_whitespaces=options.skip_whitespaces)
+        stdout.writelines(iddiff)
+    except FileNotFoundError as e:
+        stderr.write('iddiff: {}.\n'.format(e))
+        exit(2)
 
 
 if __name__ == '__main__':
